import path from "path";
import { fs as fse, getInternalPlugins } from "@modern-js/utils";
import { getProjectUsage } from "./utils";
import { handleDependencies } from "./dependencies";
var deploy_default = () => ({
  name: "@modern-js/plugin-deploy",
  pre: [
    "@modern-js/plugin-bff",
    "@modern-js/plugin-server"
  ],
  setup: (api) => {
    return {
      async beforeDeploy() {
        const appContext = api.useAppContext();
        const { appDirectory, distDirectory, serverInternalPlugins, sharedDirectory, apiDirectory, lambdaDirectory, metaName } = appContext;
        const configContext = api.useResolvedConfigContext();
        const outputDirectory = path.join(appDirectory, ".output");
        await fse.remove(outputDirectory);
        await fse.copy(distDirectory, outputDirectory, {
          filter: (src) => {
            const distStaticDirectory = path.join(distDirectory, "static");
            return !src.includes(distStaticDirectory);
          }
        });
        const { bff } = configContext;
        const config = {
          output: {
            path: "."
          },
          bff
        };
        const plugins = getInternalPlugins(appDirectory, serverInternalPlugins);
        const serverAppContext = {
          sharedDirectory: `path.join(__dirname, "${path.relative(appDirectory, sharedDirectory)}")`,
          apiDirectory: `path.join(__dirname, "${path.relative(appDirectory, apiDirectory)}")`,
          lambdaDirectory: `path.join(__dirname, "${path.relative(appDirectory, lambdaDirectory)}")`,
          metaName
        };
        const { genNodeEntry } = await import("./entrys");
        const code = genNodeEntry({
          plugins,
          config,
          appContext: serverAppContext
        });
        const { useSSR, useAPI, useWebServer } = getProjectUsage(appDirectory, outputDirectory);
        const entryFilePath = path.join(outputDirectory, "index.js");
        if (useSSR || useAPI || useWebServer) {
          await fse.writeFile(entryFilePath, code);
        }
        await handleDependencies(appDirectory, outputDirectory, [
          "@modern-js/prod-server"
        ]);
      }
    };
  }
});
export {
  deploy_default as default
};
